% function [x,res,k,flag] = cg(A,x,b,M,opts,pars)
%
%  preconditioned conjugate gradient method to solve A x = b, where A is
%  symmetric positive definite
%
%  Inputs
%    A    n by n matrix - system matrix
%         function handle to a function computing y=A*x with interface 
%            function y = Afun(x,pars)
%
%    x0   n by 1 vector an initial guess for the solution (zero works)
%
%    b    n by 1 right hand side
%
%    M    n by n matrix - preconditioner (x=M\y must be cheap)
%         function handle computing x = M\y with interface
%            function x = Mfun(x,pars)
%
%   opts  options structure with members
%    .tol     tolerance for the preconditioned residuals
%             (default 1e-6)
%
%    .maxit   maximum number of iterations
%             (default 100)
%
%   pars  [optional] user parameters to be passed to the user supplied
%         functions for the matrix-vector multiplication and preconditioner
%         application
%
% Outputs
%   x	  current iterate
%
%   res   (preconditioned) residual history
%
%   k     iteration number
%
%   flag  0 - normal termination, reached prescribed tolerance
%         1 - did not reach prescribed tolerance after maxit iter
%         2 - preconditioner M does not appear to be pos def
%         3 - matrix A does not appear to be pos def
   
% Fernando Guevara Vasquez 2008
% based on Nocedal and Wright, Numerical Optimization, Algorithm 5.3
function [x,res,k,flag] = mycg(A,x0,b,M,opts,pars)
flag=0;

if (nargin<4) opts=[]; end;
if (nargin<5) pars=[]; end;

% set default options
if (~isfield(opts,'tol')) opts.tol=1e-6; end;
if (~isfield(opts,'maxit')) opts.maxit=100; end;
tol=opts.tol; maxit=opts.maxit;

x=x0;
% compute residual
r = apply_A(A,x,pars)-b;

% apply preconditioner
y = apply_prec(M,r,pars);
p = -y;
rho = r'*y; res(1)=sqrt(rho);
if (rho<0), flag=2; return; end;
if (sqrt(rho)<=tol), flag=0; return; end;
for k=1:maxit,
 q = apply_A(A,p,pars);
 alpha = p'*q;
 if (alpha<=0), flag=3; return; end;
 alpha = rho/alpha;
 x = x + alpha*p;
 r = r + alpha*q;
 y = apply_prec(M,r,pars);
 rhonew=r'*y;
 res(k+1)=sqrt(rhonew);
 if (rhonew<0), flag=2; return; end;
 if (sqrt(rhonew)<=tol), flag=0; return; end;
 beta = rhonew/rho;
 p = -y + beta*p; 
 rho=rhonew;
end;%for k
flag=3; % maxit iterations but did not converge

% Apply A to a vector x, where A can be a function handle or a matrix
function y=apply_A(A,x,pars)
 if (isnumeric(A))
  y=A*x; 
 else
  y=A(x,pars);
 end;

% Apply preconditioner M to a vector x, where M can be a function handle, a
% matrix or empty for no preconditioning
function y=apply_prec(M,x,pars)
 if (isempty(M))
  y=x;
 else
  if (isnumeric(M))
   y=M\x;
  else
   y=M(x,pars);
  end;
 end;
