% function grid = gengrid(nb,h)
%
%  triangulates the unit circle with uniformly spaced nodes on the
%  boundary.  The grid is generated using: P.-O. Persson, G. Strang, A Simple
%  Mesh Generator in MATLAB.  SIAM Review, Volume 46 (2), pp. 329-345, June
%  2004
%
% inputs
%  nb  number of boundary nodes
%  h   approximate diameter of elements
%
% outputs a structure grid with members:
%  p   np x 2 - contains node coordinates. The first nb nodes are the
%      boundary nodes in counter clockwise ordering
%      Example:
%       p = [ x1  y1
%             x2  y2
%	        ...
%	      xnp ynp ];
%
%  t  nt x 3 - for the triangle j in the triangulation, t(j,:) are the indices
%              in p of its vertices
%
%  be nb x 2 - for the boundary edge j, be(j,:) are the indices in p of its
%              end points.

% Fernando Guevara Vasquez 2008
function grid = gengrid(nb,h)

if (nargin<2) h=0.2; end; % default value for h if not specified

fd = inline('sqrt(sum(p.^2,2))-1','p'); % distance function

T = (0:nb-1)'*2*pi/nb; pbdry = [cos(T),sin(T)]; % boundary nodes

% mesh the domain
[p,t]=distmesh2d(fd,@huniform,h,[-1,-1;1,1],pbdry);

% fix the grid, leaving all boundary nodes intact
% (removes duplicated nodes and unused nodes as well)
% This is a modified version of distmesh/fixmesh.m
pB = p(1:nb,:);
pI = p(nb+1:end,:);
snap=max(max(pI,[],1)-min(pI,[],1),[],2)*1024*eps;
[foo,ix,jx]=unique(round(pI/snap)*snap,'rows');

pI=pI(ix,:); 
p = [pB; pI]; jx = [(1:nb)'; jx+nb];
t=jx(t);

[pix,ix,jx]=unique(t);
t=reshape(jx,size(t));
p=p(pix,:);

flip=simpvol(p,t)<0;
t(flip,[1,2])=t(flip,[2,1]);


% bdry edges
be = [ (1:nb)',[2:nb,1]'];

% put everything in the output struct
grid.p = p;
grid.t = t;
grid.be = be;
